from app.utils import background_delete_job
from flask import (abort, current_app, flash, redirect, render_template,
                   send_from_directory, url_for)
from flask_login import current_user, login_required
from . import main
from ..corpora.forms import AddCorpusForm
from ..models import Job, JobInput, JobResult
import os
import threading


@main.route('/')
def index():
    return render_template('main/index.html.j2', title='Opaque')


@main.route('/dashboard')
@login_required
def dashboard():
    return render_template('main/dashboard.html.j2',
                           add_corpus_form=AddCorpusForm(), title='Dashboard')


@main.route('/jobs/<int:job_id>')
@login_required
def job(job_id):
    job = Job.query.get_or_404(job_id)
    if not (job.creator == current_user or current_user.is_administrator()):
        abort(403)
    return render_template('main/jobs/job.html.j2', job=job, title='Job')


@main.route('/jobs/<int:job_id>/delete')
@login_required
def delete_job(job_id):
    delete_thread = threading.Thread(
        target=background_delete_job,
        args=(current_app._get_current_object(), job_id)
    )
    delete_thread.start()
    flash('Job has been deleted!')
    return redirect(url_for('main.dashboard'))


@main.route('/jobs/<int:job_id>/inputs/<int:job_input_id>/download')
@login_required
def job_input_download(job_id, job_input_id):
    job_input = JobInput.query.get_or_404(job_input_id)
    if not job_input.job_id == job_id:
        abort(404)
    if not (job_input.job.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                       job_input.dir)
    return send_from_directory(as_attachment=True,
                               directory=dir,
                               filename=job_input.filename)


@main.route('/jobs/<int:job_id>/results/<int:job_result_id>/download')
@login_required
def job_result_download(job_id, job_result_id):
    job_result = JobResult.query.get_or_404(job_result_id)
    if not job_result.job_id == job_id:
        abort(404)
    if not (job_result.job.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                       job_result.dir)
    return send_from_directory(as_attachment=True,
                               directory=dir,
                               filename=job_result.filename)
